/**	TRC Magnets project, math tools [trc.magnets.math].
 * 
 *	Math classes, functions and utilities.
 *	@author Jacek "TeMPOraL" Zlydach
 *	@author www.trc.pr0.pl
 *	@version 0.1
 *	Last modification: 13.10.2008.
 */
package trc.magnets.math;

public final class Math
{
	//Vector2f - connected statics
	public static final float DotProduct(Vector2f v1, Vector2f v2)
	{
		return v1.x*v2.x + v1.y*v2.y;
	}
	public static final float Distance(Vector2f v1, Vector2f v2)
	{
		//Need to be FAST, not elegant. Hate this cloning idea... 
		return (float)java.lang.Math.sqrt( 
									(v1.x - v2.x)*(v1.x - v2.x) +	//x len
									(v1.y - v2.y)*(v1.y - v2.y) );	//y len
	}
	public static final float DistanceSqr(Vector2f v1, Vector2f v2)
	{
		return	(v1.x - v2.x)*(v1.x - v2.x) +	//x len
				(v1.y - v2.y)*(v1.y - v2.y);	//y len
	}

	//Vector3f - connected statics
	public static final float DotProduct(Vector3f v1, Vector3f v2)
	{
		return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z;
	}
	public static final float Distance(Vector3f v1, Vector3f v2)
	{
		//Need to be FAST, not elegant. Hate this cloning idea... 
		return (float)java.lang.Math.sqrt( 
									(v1.x - v2.x)*(v1.x - v2.x) +	//x len
									(v1.y - v2.y)*(v1.y - v2.y) +	//y len
									(v1.z - v2.z)*(v1.z - v2.z) );	//z len
	}
	public static final float DistanceSqr(Vector3f v1, Vector3f v2)
	{
		return	(v1.x - v2.x)*(v1.x - v2.x) +	//x len
				(v1.y - v2.y)*(v1.y - v2.y) +	//y len
				(v1.z - v2.z)*(v1.z - v2.z);	//z len
	}
	
	//constants
	public static final double PI_OVER_180 = java.lang.Math.PI / 180.0;
	public static final double PI_UNDER_180 = 180.0 / java.lang.Math.PI;
	public static final float FLOAT_EPSILON = 1e-4f;
	public static final double DOUBLE_EPSILON = 1e-8;
	
	//TODO check against compareTo() stuff from Comparable interface 
	
	public static final boolean FloatEqual(float f1, float f2)
	{
		return (java.lang.Math.abs(f1 - f2)) < FLOAT_EPSILON;
	}
	public static final boolean FloatLess(float f1, float f2)
	{
		return (f1 - f2) > FLOAT_EPSILON;
	}
	
	//clamp
	public static final float Clamp(float what, float a, float b)
	{
		return (what < a) ? a : ( (what > b) ? b : what);
	}
	
	public static float LinearInterpolate(float a, float b, float step)
	{
		float _step = Clamp(step, 0, 1);
		
		return b*_step + a*(1.0f-_step);
	}

	public static void main(String[] args)
	{
		//TODO Tests of general math functions go here:
		
		//use case of vector 2
		Vector2f v1 = new Vector2f(100.0f, 100.0f);
		Vector2f v2 = new Vector2f(0.0f, 1.0f);
		Vector2f v3 = new Vector2f();
		
		v3.SetToSum(v1, v2);
		System.out.println("v3: " + v3);
		v3.SetToDifference(v2, v1);
		System.out.println("v3: " + v3);
		System.out.println("Len(v3): " + v3.Length());
		System.out.println("Len(v2): " + v2.Length());
		System.out.println("Len(v1): " + v1.Length());
		System.out.println("Distance(v1,v2): " + Distance(v1,v2));
		v1.Normalize();
		System.out.println("v1: " + v1 + "; Length: " + v1.Length());
		//simple calculations
		v1.ScaleBy(100.0f);
		System.out.println("v1: " + v1 + "; Length: " + v1.Length());
		v2.Zero();
		System.out.println("v2 zeroed, len: " + v2.Length());
		v2.SetLength(1.0f);
		System.out.println("v2 unzeroed, len: " + v2.Length());
		
		//do some real vector math though
		float mass = 100.0f;
		float dT = 1.0f/60.0f;
		Vector3f pos = new Vector3f(100f, 100f, 0.0f);
		Vector3f force = new Vector3f(100.0f, 0.0f, -50.0f);
		Vector3f vel = new Vector3f(0.0f, 0.0f, 4.0f);
		Vector3f prevVel = new Vector3f();
		
		//step 40 times
		prevVel.SetTo(vel);
		for(int i = 0 ; i < 200 ; ++i)
		{
			//x = x_0 + v0dt + at^2/2
			pos.AddScaled(vel, dT);	//x_1 = x_0 + v0dt;
			pos.AddScaled(force, (dT*dT)/(2*mass));	//x_n = x_1 + at^2/2
			vel.AddScaled(force, (dT)/mass);	//v_n = v_0 + adt;
			System.out.println(i + ") pos=" + pos + "; vel = " + vel + " [dv/dt = " + (vel.Length() - prevVel.Length()) + "; force = " + force);
			prevVel.SetTo(vel);
		}
		
		
	}

}
