/**	TRC Magnets project, math tools. [trc.magnets.math].
 * 
 * 	2D Vector class with floating-point components for fast computation.
 *	@author Jacek "TeMPOraL" Zlydach
 *	@author www.trc.pr0.pl
 *	@version 0.1
 *	Last modification: 13.10.2008.
 */

/**
 * TODO Comment all the stuff with javadoc-style comments.
 * TODO Make function names begin with lowercase letters.
 */
package trc.magnets.math;

/**
 * @author TeMPOraL
 *
 */
public final class Vector2f
{
	//variables
	public float x;
	public float y;
	
	//methods
	public Vector2f()
	{
	}
	public Vector2f(float _x, float _y)
	{
		SetTo(_x,_y);
	}
	
	//math operations
	public final void SetTo(float _x, float _y)
	{
		x = _x;
		y = _y;
	}
	public final void SetTo(Vector2f what)
	{
		SetTo(what.x, what.y);
	}
	public final void SetToScaled(Vector2f what, float scale)
	{
		SetTo(what.x*scale, what.y*scale);
	}
	public final void SetToSum(Vector2f v1, Vector2f v2)
	{
		x = v1.x + v2.x;
		y = v1.y + v2.y;
	}
	public final void SetToDifference(Vector2f v1, Vector2f v2)
	{
		x = v1.x - v2.x;
		y = v1.y - v2.y;
	}
	
	public final void SetToComponentProduct(Vector2f v1, Vector2f v2)
	{
		x = v1.x * v2.x;
		y = v1.y * v2.y;
	}
	
	public final void Add(Vector2f what)
	{
		x += what.x;
		y += what.y;
	}
	public final void AddScaled(float _x, float _y, float scale)
	{
		x += _x*scale;
		y += _y*scale;
	}
	public final void AddScaled(Vector2f what, float scale)
	{
		x += what.x*scale;
		y += what.y*scale;
	}
	public final void Substract(Vector2f what)
	{
		x -= what.x;
		y -= what.y;
	}
	public final void SubstractScaled(Vector2f what, float scale)
	{
		x -= what.x*scale;
		y -= what.y*scale;
	}
	
	public final void ScaleBy(float scalar)
	{
		x *= scalar;
		y *= scalar;
	}
	
	public final float Length()
	{
		return (float)java.lang.Math.sqrt(x*x + y*y);
	}
	public final float LengthSqr()
	{
		return x*x + y*y;
	}
	
	public final void SetLength(float newLen)
	{
		ScaleBy(newLen/Length());
	}
	public final void Normalize()
	{
		ScaleBy(1/Length());
	}
	public final void NormalizeSafe()
	{
		if(LengthSqr() > trc.magnets.math.Math.FLOAT_EPSILON)
		{
			Normalize();
			return;
		}
		//arbitrary values
		x = 0.0f;
		y = 0.0f;
	}
	public final void Zero()
	{
		x = y = 0.0f;
	}
	public final void Negate()
	{
		x = -x;
		y = -y;
	}
	public final String toString()
	{
		return "(" + x + ", " + y + ")";
	}
}
