/**	TRC Magnets project, math tools [trc.magnets.math].
 * 
 * 	3D Vector class with floating-point components for fast computation. 
 *	@author Jacek "TeMPOraL" Zlydach
 *	@author www.trc.pr0.pl
 *	@version 0.1
 *	Last modification: 2008-10-13.
 */
package trc.magnets.math;

/**
 * @author TeMPOraL
 *
 */
public final class Vector3f
{
	//variables
	public float x;
	public float y;
	public float z;
	
	//methods
	public Vector3f()
	{
	}
	public Vector3f(float _x, float _y, float _z)
	{
		SetTo(_x,_y,_z);
	}
	
	//math operations
	public final void SetTo(float _x, float _y, float _z)
	{
		x = _x;
		y = _y;
		z = _z;
	}
	public final void SetTo(Vector3f what)
	{
		SetTo(what.x, what.y, what.z);
	}
	public final void SetToScaled(Vector3f what, float scale)
	{
		SetTo(what.x*scale, what.y*scale, what.z*scale);
	}
	public final void SetToSum(Vector3f v1, Vector3f v2)
	{
		x = v1.x + v2.x;
		y = v1.y + v2.y;
		z = v1.z + v2.z;
	}
	public final void SetToDifference(Vector3f v1, Vector3f v2)
	{
		x = v1.x - v2.x;
		y = v1.y - v2.y;
		z = v1.z - v2.z;
	}
	
	public final void SetToCrossProduct(Vector3f v1, Vector3f v2)
	{
		SetTo((v1.y * v2.z) - (v1.z * v2.y), (v1.z * v2.x) - (v1.x - v2.z),
				(v1.x * v2.y) - (v1.y - v2.x) );
	}
	
	public final void SetToComponentProduct(Vector3f v1, Vector3f v2)
	{
		x = v1.x * v2.x;
		y = v1.y * v2.y;
		z = v1.z * v2.z;
	}
	
	public final void Add(Vector3f what)
	{
		x += what.x;
		y += what.y;
		z += what.z;
	}
	public final void AddScaled(Vector3f what, float scale)
	{
		x += what.x*scale;
		y += what.y*scale;
		z += what.z*scale;
	}
	public final void Substract(Vector3f what)
	{
		x -= what.x;
		y -= what.y;
		z -= what.z;
	}
	public final void SubstractScaled(Vector3f what, float scale)
	{
		x -= what.x*scale;
		y -= what.y*scale;
		z -= what.z*scale;
	}
	
	public final void ScaleBy(float scalar)
	{
		x *= scalar;
		y *= scalar;
		z *= scalar;
	}
	
	public final float Length()
	{
		return (float)java.lang.Math.sqrt(x*x + y*y + z*z);
	}
	public final float LengthSqr()
	{
		return x*x + y*y + z*z;
	}
	
	public final void SetLength(float newLen)
	{
		ScaleBy(newLen/Length());
	}
	public final void Normalize()
	{
		ScaleBy(1/Length());
	}
	public final void Zero()
	{
		x = y = z = 0.0f;
	}
	public final void Negate()
	{
		x = -x;
		y = -y;
		z = -z;
	}
	public final String toString()
	{
		return "(" + x + ", " + y + ", " + z + ")";
	}
}
