package trc.magnets.sunray;

import java.util.LinkedList;

import trc.magnets.math.Vector2f;

public class FlockAI
{
	PlayerCow player;	//store player data
	
	LinkedList<Entity> entities = new LinkedList<Entity>();	//who do we have
	
	Vector2f tmpVec = new Vector2f(0.0f, 0.0f);
	Vector2f tmpVec2 = new Vector2f(0.0f, 0.0f);
	Vector2f commonGoal = new Vector2f(200.0f, 400.0f);
	
	Vector2f tempCoM = new Vector2f(0.0f, 0.0f);	//center of mass
	
	//priorities
	final static float STEER_TO_CENTER_OF_MASS_PRIORITY = 2.0f;
	final static float AVOID_PLAYER_PRIORITY = 2.0f;
	final static float AVOID_COLLISION_PRIORITY = 5.0f;
	final static float STEER_TO_COMMON_GOAL_PRIORITY = 3.0f;
	
	final static float CENTER_OF_MASS_TOLERANCE = 0.0f;
	final static float PLAYER_DETECT_RANGE = 80.0f;
	final static float COLLISION_DETECT_RANGE = 80.0f;
	final static float COMMON_GOAL_TOLERANCE = 50.0f;
	
	FlockAI(PlayerCow playerRef)
	{
		player = playerRef;
	}
	
	public void AcquireNewVelocity(Entity whoAsks)
	{
		//4 rules:
		// * integrity (steer to center of mass)
		// * avoid player
		// * avoid bumping at each other
		// * common goal
		
		whoAsks.velocity.Zero();
		tmpVec.Zero();
		
		ComputeRule_CenterOfMass(whoAsks, tmpVec);
		tmpVec.NormalizeSafe();
		tmpVec.ScaleBy(STEER_TO_CENTER_OF_MASS_PRIORITY);
		whoAsks.velocity.SetTo(tmpVec);
		
		ComputeRule_AvoidPlayer(whoAsks, tmpVec);
		tmpVec.NormalizeSafe();
		whoAsks.velocity.AddScaled(tmpVec, AVOID_PLAYER_PRIORITY);
		
		ComputeRule_AvoidCollision(whoAsks, tmpVec);
		tmpVec.NormalizeSafe();
		whoAsks.velocity.AddScaled(tmpVec, AVOID_COLLISION_PRIORITY);
		
		ComputeRule_CommonGoal(whoAsks, tmpVec);
		tmpVec.NormalizeSafe();
		whoAsks.velocity.AddScaled(tmpVec, STEER_TO_COMMON_GOAL_PRIORITY);
		
		whoAsks.velocity.NormalizeSafe();
		whoAsks.velocity.ScaleBy(Entity.ENTITY_SPEED);
		
	} 
	
	void ComputeRule_CenterOfMass(Entity whoAsks, Vector2f tmpVec)
	{
		if(trc.magnets.math.Math.DistanceSqr(whoAsks.position, tempCoM) < CENTER_OF_MASS_TOLERANCE*CENTER_OF_MASS_TOLERANCE)
		{
			tmpVec.SetTo(tempCoM.x - whoAsks.position.x, tempCoM.y - whoAsks.position.y);
		}
	}
	
	void ComputeRule_AvoidPlayer(Entity whoAsks, Vector2f tmpVec)
	{
		if(trc.magnets.math.Math.DistanceSqr(whoAsks.position, player.position) < PLAYER_DETECT_RANGE*PLAYER_DETECT_RANGE)
		{
			tmpVec.SetTo(whoAsks.position.x - player.position.x, whoAsks.position.y - player.position.y);
		}
	}
	
	void ComputeRule_AvoidCollision(Entity whoAsks, Vector2f tmpVec)
	{
		tmpVec2.Zero();
		for(Entity pig : entities)
		{
			if(pig != whoAsks)
			{
				if(trc.magnets.math.Math.DistanceSqr(whoAsks.position, pig.position) < COLLISION_DETECT_RANGE*COLLISION_DETECT_RANGE)
				{
					tmpVec2.SetTo(whoAsks.position.x - pig.position.x, whoAsks.position.y - pig.position.y);
					tmpVec2.NormalizeSafe();
					tmpVec.Add(tmpVec2);
				}
			}
		}
	}
	
	void ComputeRule_CommonGoal(Entity whoAsks, Vector2f tmpVec)
	{
		if(trc.magnets.math.Math.DistanceSqr(whoAsks.position, commonGoal) > COMMON_GOAL_TOLERANCE*COMMON_GOAL_TOLERANCE)
		{
			tmpVec.SetTo(commonGoal.x - whoAsks.position.x, commonGoal.y - whoAsks.position.y);
		}
	}
	
	void AddEntity(Entity who)
	{
		entities.add(who);
	}
	
	void Update(float dT)
	{
		//
		tempCoM.Zero();
		for(Entity pig : entities)
		{
			pig.Update(dT);
			tempCoM.Add(pig.position);
		}
		tempCoM.ScaleBy(entities.size());
	}
	
	void RenderAll()
	{
		for(Entity pig : entities)
		{
			pig.RenderSelf();
		}
	}
}