package trc.magnets.sunray;

import java.io.IOException;

import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.GL11;
import org.lwjgl.util.Color;

import trc.magnets.math.Vector2f;

public class PlayerCow
{
	//==== data
	Vector2f position = new Vector2f(100.0f, 200.0f);
	Vector2f size = new Vector2f(40.0f, 40.0f);
	
	Vector2f emotions = new Vector2f(0.0f, 0.0f);	//happy - unhappy, afraid - overjoyed
	
	Vector2f destination =  new Vector2f(0.0f, 0.0f);	//where to go?
	Vector2f velocity = new Vector2f(0.0f, 0.0f);	//just a normal velocity ;)
	
	static final Vector2f COW_HAPPY = new Vector2f(1.0f, 0.0f);
	static final Vector2f COW_SAD = new Vector2f(-1.0f, 0.0f);
	static final Vector2f COW_ANGRY = new Vector2f(0.0f, -1.0f);
	static final Vector2f COW_JOYED = new Vector2f(0.0f, 1.0f);
	
	static final float PIG_FUN_DIST = 90.0f;
	
	static final Vector2f DEFAULT_EMOTIONAL_DRAG = new Vector2f(-0.3f, 0.01f);
	static final Vector2f PORK_DRAG = new Vector2f(0.05f, 0.5f);
	static final Vector2f GOOD_GRASS_DRAG = new Vector2f(0.8f, 0.0f);
	
	static final float GOOD_GRASS_RANGE = 175.0f;
	static final Vector2f GOOD_GRASS_POS = new Vector2f(525.0f, 325.0f);
	
	///nana nana nana, little kitty mine makes me feel so fine ;)
	
	Color color = (Color) Color.WHITE;
	
	static final float DESTINATION_TOLERANCE = 5.0f;	//in px, when to stop going
	static final float COW_SPEED = 100.0f;
	
	//==== display data
	STextureData cowTexture;
	
	//==== UI display data
	STextureData cowVeryHappy;
	STextureData cowHappy;
	STextureData cowSad;
	STextureData cowVerySad;
	
	STextureData cowMad;
	STextureData cowJoyed;
	
	STextureData cowDefault;
	
	STextureData currentFace;
	
	//==== controll
	boolean bButtonPressedLastFrame = false;
	
	PlayerCow()
	{
		// ustaw parametry

		// zaladuj tekstury
		try
		{
			cowTexture = Main.textureLoader.LoadTexture("data/cow.png");
			
			cowHappy = Main.textureLoader.LoadTexture("data/cow-face happy.png");
			cowVeryHappy = Main.textureLoader.LoadTexture("data/cow-face vhappy.png");
			cowSad = Main.textureLoader.LoadTexture("data/cow-face sad.png");
			cowVerySad = Main.textureLoader.LoadTexture("data/cow-face vsad.png");
			cowMad = Main.textureLoader.LoadTexture("data/cow-face mad.png");
			cowJoyed = Main.textureLoader.LoadTexture("data/cow-face joy.png");
			cowDefault = cowHappy;
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
	
	void Update(float dT)
	{
		//recompute velocity each frame
		if(trc.magnets.math.Math.DistanceSqr(position, destination) > DESTINATION_TOLERANCE*DESTINATION_TOLERANCE)
		{
			velocity.SetTo(destination.x - position.x, destination.y - position.y);
			velocity.Normalize();
			velocity.ScaleBy(COW_SPEED);
		}
		else
		{
			velocity.Zero();
		}
		
		//update position
		position.AddScaled(velocity, dT);
		
		//emotions
		
		//	ensure <-2 : 2>
		emotions.x = trc.magnets.math.Math.Clamp(emotions.x, -2.0f, 2.0f);
		emotions.y = trc.magnets.math.Math.Clamp(emotions.y, -2.0f, 2.0f);
		
		//	switch GFX
		//overjoyed
		if(emotions.y > 1.5f)
		{
			//System.out.println("cow joyed");
			currentFace = cowJoyed;
		}
		else if(emotions.y < -1.5f)
		{
			//System.out.println("cow mad");
			currentFace = cowMad;
		}
		//normal emotions
		else if(emotions.x < -1.5f)
		{
			//System.out.println("cow very sad");
			currentFace = cowVerySad;
		}
		else if(emotions.x < -0.5f)
		{
			//System.out.println("cow sad");
			currentFace = cowSad;
		}
		else if(emotions.x > 1.5f)
		{
			//System.out.println("cow very happy");
			currentFace = cowVeryHappy;
		}
		else if(emotions.x > 0.5f)
		{
			//System.out.println("cow happy");
			currentFace = cowHappy;
		}
		else
		{
			//System.out.println("cow default");
			currentFace = cowDefault;
		}
		cowTexture = currentFace;
		
		//default emotion drag
		emotions.AddScaled(DEFAULT_EMOTIONAL_DRAG, dT);
		
		for(Entity pig : Main.pigs.entities)
		{
			if(trc.magnets.math.Math.DistanceSqr(position, pig.position) < PIG_FUN_DIST*PIG_FUN_DIST)
			{
				emotions.AddScaled(PORK_DRAG, dT);
			}
		}
		if(emotions.y > 1.0f)
		{
			emotions.y -= 1.0f*dT;
		}
		
		//near good grass
		if(trc.magnets.math.Math.DistanceSqr(position, GOOD_GRASS_POS) < GOOD_GRASS_RANGE*GOOD_GRASS_RANGE)
		{
			emotions.AddScaled(GOOD_GRASS_DRAG, dT);
			//System.out.println("good grass range");
		}
		
		//controll
		if(Mouse.isButtonDown(0) && !bButtonPressedLastFrame)
		{
			bButtonPressedLastFrame = true;
			destination.SetTo(Mouse.getX(), Mouse.getY());
		}
		bButtonPressedLastFrame = Mouse.isButtonDown(0);	//save last state
	}
	
	void RenderSelf()
	{
		GL11.glColor4ub(color.getRedByte(), color.getGreenByte(), color.getBlueByte(), color.getAlphaByte());
		//draw a quad
		Main.DrawQuad(position, size, cowTexture);
	}
	
}
