/* Ponizszy fragment zostal przygotowany przed Compo.
 * Zawiera on jedynie kod potrzebny do wczytania obrazka z dysku
 * i zrobienia z niego tekstury w OpenGL. Zadnego zarzadzania
 * zasobami, etc. - zwykla zamiana obrazka na texture id ;).
 * 
 * Sam kod jest na bazie loaderow tekstur z przykladow LWJGL
 * oraz z gry Prototyp (www.google.pl?q=lwjgl+prototyp)
 * */

package trc.magnets.sunray;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.ComponentColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.DataBufferByte;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;
import java.util.Hashtable;

import javax.imageio.ImageIO;

import org.lwjgl.BufferUtils;
import org.lwjgl.opengl.GL11;

class STextureData
{
	int width;
	int height;
	int handle;
	int textureID;
}

public class TextureLoader
{
    /** The colour model including alpha for the GL image */
    private ColorModel glAlphaColorModel;
    
    /** The colour model for the GL image */
    private ColorModel glColorModel;
    
    /** Scratch buffer for texture ID's */
    private IntBuffer textureIDBuffer = BufferUtils.createIntBuffer(1);
    
    //c-tor
    TextureLoader()
    {
		//init texture loader stuff
		glAlphaColorModel = new ComponentColorModel(ColorSpace.getInstance(ColorSpace.CS_sRGB),
                new int[] {8,8,8,8},
                true,
                false,
                ComponentColorModel.TRANSLUCENT,
                DataBuffer.TYPE_BYTE);
                
		glColorModel = new ComponentColorModel(ColorSpace.getInstance(ColorSpace.CS_sRGB),
                new int[] {8,8,8,0},
                false,
                false,
                ComponentColorModel.OPAQUE,
                DataBuffer.TYPE_BYTE);
    }
    
	int GenerateTextureID()
	{
		GL11.glGenTextures(textureIDBuffer); 
		return textureIDBuffer.get(0);
	}
	
	/*
	 * Function for external world.
	 * */
	public STextureData LoadTexture(String textureName) throws IOException
	{
		int srcPixelFormat = 0;

		// create the texture ID for this texture 
		int textureID = GenerateTextureID();
		
		STextureData texture = new STextureData();
		
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, textureID);
		
		BufferedImage bufferedImage = LoadImage(textureName);
		texture.width = bufferedImage.getWidth();
		texture.height = bufferedImage.getHeight();
		texture.textureID = textureID;
		
		if (bufferedImage.getColorModel().hasAlpha())
		{
			srcPixelFormat = GL11.GL_RGBA;
		}
		else
		{
			srcPixelFormat = GL11.GL_RGB;
		}

		// convert that image into a byte buffer of texture data 
		ByteBuffer textureBuffer = ConvertImageData(bufferedImage,texture); 
	        

		//FIXME move min/mag filters to somewhere else
		GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_LINEAR); 
		GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_LINEAR);  

		//produce a texture from the byte buffer
		GL11.glTexImage2D(GL11.GL_TEXTURE_2D, 
							0, 
							GL11.GL_RGBA,	//FIXME destPixelFormat - move somewhere else 
							Get2Fold(bufferedImage.getWidth()), 
							Get2Fold(bufferedImage.getHeight()), 
							0, 
							srcPixelFormat, 
							GL11.GL_UNSIGNED_BYTE, 
							textureBuffer ); 
	        
		return texture; 
	}
	
	int Get2Fold(int fold)
	{
		int ret = 2;
		while (ret < fold)
		{
			ret *= 2;
		}
		return ret;
	}
	
	BufferedImage LoadImage(String ref) throws IOException
	{ 
		URL url = TextureLoader.class.getClassLoader().getResource(ref);
		
        if (url == null)
        {
        	throw new IOException("Cannot find: " + ref);
        }
        
        BufferedImage bufferedImage = ImageIO.read(new BufferedInputStream(getClass().getClassLoader().getResourceAsStream(ref))); 
 
        return bufferedImage;
	}
	
    @SuppressWarnings("unchecked")
	private ByteBuffer ConvertImageData(BufferedImage bufferedImage,STextureData texture) { 
		ByteBuffer imageBuffer = null; 
		WritableRaster raster;
		BufferedImage texImage;
		
		int texWidth = 2;
		int texHeight = 2;
		
		// find the closest power of 2 for the width and height
		// of the produced texture
		while (texWidth < bufferedImage.getWidth()) {
		    texWidth *= 2;
		}
		while (texHeight < bufferedImage.getHeight()) {
		    texHeight *= 2;
		}
		
		texture.height = texHeight;
		texture.width = texWidth;
		
		// create a raster that can be used by OpenGL as a source
		// for a texture
		if (bufferedImage.getColorModel().hasAlpha())
		{
			raster = Raster.createInterleavedRaster(DataBuffer.TYPE_BYTE,texWidth,texHeight,4,null);
			texImage = new BufferedImage(glAlphaColorModel,raster,false,new Hashtable());
		}
		else
		{
			raster = Raster.createInterleavedRaster(DataBuffer.TYPE_BYTE,texWidth,texHeight,3,null);
			texImage = new BufferedImage(glColorModel,raster,false,new Hashtable());
		}
            
		// copy the source image into the produced image
		Graphics g = texImage.getGraphics();
		g.setColor(new Color(0f,0f,0f,0f));
		g.fillRect(0,0,texWidth,texHeight);
		g.drawImage(bufferedImage,0,0,null);
		
		// build a byte buffer from the temporary image 
		// that be used by OpenGL to produce a texture.
		byte[] data = ((DataBufferByte) texImage.getRaster().getDataBuffer()).getData(); 

		imageBuffer = ByteBuffer.allocateDirect(data.length); 
		imageBuffer.order(ByteOrder.nativeOrder()); 
		imageBuffer.put(data, 0, data.length); 
		imageBuffer.flip();
		
		return imageBuffer; 
	}
	
}
